@@ -2,7 +2,7 @@
  *
  * Copyright © 2004-2006 Bastien Nocera <hadess@hadess.net>
  * Copyright © 2002 David A. Schleef <ds@schleef.org>
- * Copyright © 2006 Christian Persch
+ * Copyright © 2006, 2009 Christian Persch
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -76,7 +76,6 @@ char * totem_pl_parser_resolve_uri (GFile *base_gfile, const char *relative_uri)
 
 typedef enum {
 	TOTEM_PLUGIN_TYPE_GMP,
-	TOTEM_PLUGIN_TYPE_COMPLEX,
 	TOTEM_PLUGIN_TYPE_NARROWSPACE,
 	TOTEM_PLUGIN_TYPE_MULLY,
 	TOTEM_PLUGIN_TYPE_CONE,
@@ -94,6 +93,7 @@ typedef GObjectClass TotemEmbeddedClass;
 
 typedef struct {
 	char *uri;
+	char *subtitle;
 	char *title;
 	int duration;
 	int starttime;
@@ -112,8 +112,10 @@ typedef struct _TotemEmbedded {
 	int width, height;
         char *user_agent;
 	const char *mimetype;
+        char *referrer_uri;
 	char *base_uri;
 	char *current_uri;
+	char *current_subtitle_uri;
 	char *href_uri;
 	char *target;
 	char *stream_uri;
@@ -189,6 +191,8 @@ void on_preferences1_activate (GtkButton *button, TotemEmbedded *emb);
 void on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb);
 void on_fullscreen1_activate (GtkMenuItem *menuitem, TotemEmbedded *emb);
 
+static void update_fill (TotemEmbedded *emb, gdouble level);
+
 enum {
 	BUTTON_PRESS,
 	START_STREAM,
@@ -290,15 +294,10 @@ totem_embedded_error_and_exit (char *title, char *reason, TotemEmbedded *emb)
 }
 
 static void
-totem_embedded_save_volume (TotemEmbedded *emb, double volume)
+totem_embedded_volume_changed (TotemEmbedded *emb, double volume)
 {
-	GConfClient *gc;
 	GValue value = { 0, };
 
-	gc = gconf_client_get_default ();
-	gconf_client_set_int (gc, GCONF_PREFIX"/volume", (int) (volume * 100.0), NULL);
-	g_object_unref (G_OBJECT (gc));
-
 	g_value_init (&value, G_TYPE_DOUBLE);
 	g_value_set_double (&value, volume);
 	g_signal_emit (emb, signals[PROPERTY_CHANGE], 0,
@@ -424,13 +423,14 @@ totem_embedded_open_internal (TotemEmbedded *emb,
 		return FALSE;
 	}
 
-	g_message ("totem_embedded_open_internal '%s' is-browser-stream %d start-play %d", uri, emb->is_browser_stream, start_play);
+	g_message ("totem_embedded_open_internal '%s' subtitle '%s' is-browser-stream %d start-play %d",
+		   uri, emb->current_subtitle_uri, emb->is_browser_stream, start_play);
 
 	bacon_video_widget_set_logo_mode (emb->bvw, FALSE);
 
 	/* FIXME: remove |err| and rely on async on_error? */
 	g_message ("BEFORE _open");
-	retval = bacon_video_widget_open (emb->bvw, uri, NULL, &err);
+	retval = bacon_video_widget_open (emb->bvw, uri, emb->current_subtitle_uri, &err);
 	g_message ("AFTER _open (ret: %d)", retval);
 	if (retval == FALSE)
 	{
@@ -527,8 +527,6 @@ totem_embedded_do_command (TotemEmbedded *embedded,
 {
 	g_return_val_if_fail (command != NULL, FALSE);
 
-	g_message ("totem_embedded_do_command: %s", command);
-
 	if (strcmp (command, TOTEM_COMMAND_PLAY) == 0) {
 		return totem_embedded_play (embedded, error);
 	}
@@ -552,9 +550,6 @@ totem_embedded_set_href (TotemEmbedded *embedded,
 			 const char *target,
 			 GError *error)
 {
-	g_message ("totem_embedded_set_href %s (target: %s)",
-		   href_uri, target);
-
 	g_free (embedded->href_uri);
 	g_free (embedded->target);
 
@@ -583,7 +578,7 @@ totem_embedded_set_volume (TotemEmbedded *embedded,
 {
 	g_message ("totem_embedded_set_volume: %f", volume);
 	bacon_video_widget_set_volume (embedded->bvw, volume);
-	totem_embedded_save_volume (embedded, volume);
+	totem_embedded_volume_changed (embedded, volume);
 	return TRUE;
 }
 
@@ -626,13 +621,15 @@ static void
 totem_embedded_set_uri (TotemEmbedded *emb,
 		        const char *uri,
 		        const char *base_uri,
+		        const char *subtitle,
 		        gboolean is_browser_stream)
 {
 	GFile *base_gfile;
-	char *old_uri, *old_base, *old_href;
+	char *old_uri, *old_base, *old_href, *old_subtitle;
 
 	base_gfile = NULL;
 	old_uri = emb->current_uri;
+	old_subtitle = emb->current_subtitle_uri;
 	old_base = emb->base_uri;
 	old_href = emb->href_uri;
 
@@ -640,13 +637,18 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	if (base_uri)
 		base_gfile = g_file_new_for_uri (base_uri);
 	emb->current_uri = totem_pl_parser_resolve_uri (base_gfile, uri);
+	if (subtitle != NULL)
+		emb->current_subtitle_uri = totem_pl_parser_resolve_uri (base_gfile, subtitle);
+	else
+		emb->current_subtitle_uri = NULL;
 	if (base_gfile)
 		g_object_unref (base_gfile);
 	emb->is_browser_stream = (is_browser_stream != FALSE);
 	emb->href_uri = NULL;
 
 	if (uri != NULL)
-		g_print ("totem_embedded_set_uri uri %s base %s => resolved %s\n", uri, base_uri, emb->current_uri);
+		g_print ("totem_embedded_set_uri uri %s base %s => resolved %s (subtitle %s => resolved %s)\n",
+			 uri, base_uri, emb->current_uri, subtitle, emb->current_subtitle_uri);
 	else
 		g_print ("Emptying current_uri\n");
 
@@ -659,6 +661,7 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	g_free (old_uri);
 	g_free (old_base);
 	g_free (old_href);
+	g_free (old_subtitle);
 	g_free (emb->stream_uri);
 	emb->stream_uri = NULL;
 }
@@ -682,14 +685,13 @@ totem_pl_item_free (gpointer data, gpointer user_data)
 		return;
 	g_free (item->uri);
 	g_free (item->title);
-	g_free (item);
+	g_free (item->subtitle);
+	g_slice_free (TotemPlItem, item);
 }
 
 static gboolean
 totem_embedded_clear_playlist (TotemEmbedded *emb, GError *error)
 {
-	g_message ("totem_embedded_clear_playlist");
-
 	g_list_foreach (emb->playlist, (GFunc) totem_pl_item_free, NULL);
 	g_list_free (emb->playlist);
 
@@ -697,23 +699,31 @@ totem_embedded_clear_playlist (TotemEmbedded *emb, GError *error)
 	emb->current = NULL;
 	emb->num_items = 0;
 
-	totem_embedded_set_uri (emb, NULL, NULL, FALSE);
+	totem_embedded_set_uri (emb, NULL, NULL, NULL, FALSE);
 
 	bacon_video_widget_close (emb->bvw);
+	update_fill (emb, -1.0);
 	totem_embedded_update_title (emb, NULL);
 
 	return TRUE;
 }
 
 static gboolean
-totem_embedded_add_item (TotemEmbedded *embedded, const char *uri, GError *error)
+totem_embedded_add_item (TotemEmbedded *embedded,
+			 const char *base_uri,
+			 const char *uri,
+			 const char *title,
+			 const char *subtitle,
+			 GError *error)
 {
 	TotemPlItem *item;
 
 	g_message ("totem_embedded_add_item: %s", uri);
 
-	item = g_new0 (TotemPlItem, 1);
+	item = g_slice_new0 (TotemPlItem);
 	item->uri = g_strdup (uri);
+	item->title = g_strdup (title);
+	item->subtitle = g_strdup (subtitle);
 	item->duration = -1;
 	item->starttime = -1;
 
@@ -724,7 +734,8 @@ totem_embedded_add_item (TotemEmbedded *embedded, const char *uri, GError *error
 		embedded->current = embedded->playlist;
 		totem_embedded_set_uri (embedded,
 					(const char *) uri,
-					embedded->base_uri /* FIXME? */,
+					base_uri,
+					subtitle,
 					FALSE);
 		totem_embedded_open_internal (embedded, FALSE, NULL /* FIXME */);
 	}
@@ -798,7 +809,7 @@ totem_embedded_open_uri (TotemEmbedded *emb,
 
 	totem_embedded_clear_playlist (emb, NULL);
 
-	totem_embedded_set_uri (emb, uri, base_uri, FALSE);
+	totem_embedded_set_uri (emb, uri, base_uri, NULL, FALSE);
 	/* We can only have one item in the "playlist" when
 	 * we open a particular URI like this */
 	emb->num_items = 1;
@@ -816,7 +827,7 @@ totem_embedded_open_stream (TotemEmbedded *emb,
 
 	totem_embedded_clear_playlist (emb, NULL);
 
-	totem_embedded_set_uri (emb, uri, base_uri, TRUE);
+	totem_embedded_set_uri (emb, uri, base_uri, NULL, TRUE);
 	/* We can only have one item in the "playlist" when
 	 * we open a browser stream */
 	emb->num_items = 1;
@@ -830,13 +841,12 @@ static gboolean
 totem_embedded_close_stream (TotemEmbedded *emb,
 			     GError *error)
 {
-	g_message ("totem_embedded_close_stream");
-
 	if (!emb->is_browser_stream)
 		return TRUE;
 
 	/* FIXME this enough? */
 	bacon_video_widget_close (emb->bvw);
+	update_fill (emb, -1.0);
 
 	return TRUE;
 }
@@ -866,9 +876,13 @@ totem_embedded_open_playlist_item (TotemEmbedded *emb,
 	totem_embedded_set_uri (emb,
 				(const char *) plitem->uri,
 			        emb->base_uri /* FIXME? */,
+			        plitem->subtitle,
 			        FALSE);
 
 	bacon_video_widget_close (emb->bvw);
+	update_fill (emb, -1.0);
+
+	//FIXME set the title from the URI if possible
 	totem_embedded_update_title (emb, plitem->title);
 	if (totem_embedded_open_internal (emb, FALSE, NULL /* FIXME */)) {
 		if (plitem->starttime > 0) {
@@ -899,8 +913,7 @@ totem_embedded_set_local_file (TotemEmbedded *emb,
 {
 	char *file_uri;
 
-	g_message ("Setting the current path to %s (uri: %s base: %s)",
-		   path, uri, base_uri);
+	g_message ("Setting the current path to %s", path);
 
 	totem_embedded_clear_playlist (emb, NULL);
 
@@ -909,7 +922,7 @@ totem_embedded_set_local_file (TotemEmbedded *emb,
 		return FALSE;
 
 	/* FIXME what about |uri| param?!! */
-	totem_embedded_set_uri (emb, file_uri, base_uri, FALSE);
+	totem_embedded_set_uri (emb, file_uri, base_uri, emb->current_subtitle_uri, FALSE);
 	g_free (file_uri);
 
 	return totem_embedded_open_internal (emb, TRUE, error);
@@ -922,8 +935,6 @@ totem_embedded_set_local_cache (TotemEmbedded *emb,
 {
 	int fd;
 
-	g_message ("totem_embedded_set_local_cache: %s", path);
-
 	/* FIXME Should also handle playlists */
 	if (!emb->is_browser_stream)
 		return TRUE;
@@ -956,8 +967,8 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	GFile *src, *dst;
 	int fd;
 
-	g_message ("Setting the current playlist to %s (uri: %s base: %s)",
-		   path, uri, base_uri);
+	g_message ("Setting the current playlist to %s (base: %s)",
+		   path, base_uri);
 
 	totem_embedded_clear_playlist (emb, NULL);
 
@@ -975,7 +986,7 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	}
 	src = g_file_new_for_path (path);
 	dst = g_file_new_for_path (tmpfile);
-	if (g_file_copy (src, dst, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &err) == FALSE) {
+	if (g_file_copy (src, dst, G_FILE_COPY_OVERWRITE | G_FILE_COPY_TARGET_DEFAULT_PERMS, NULL, NULL, NULL, &err) == FALSE) {
 		g_warning ("Failed to copy playlist '%s' to '%s': %s",
 			   path, tmpfile, err->message);
 		g_error_free (err);
@@ -993,7 +1004,7 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	g_object_unref (dst);
 	close (fd);
 
-	totem_embedded_set_uri (emb, file_uri, base_uri, FALSE);
+	totem_embedded_set_uri (emb, file_uri, base_uri, NULL, FALSE);
 	g_free (file_uri);
 
 	/* Schedule parsing on idle */
@@ -1156,9 +1167,7 @@ on_copy_location1_activate (GtkButton *button, TotemEmbedded *emb)
 	GtkClipboard *clip;
 	const char *uri;
 
-	if (emb->stream_uri != NULL) {
-		uri = emb->stream_uri;
-	} else if (emb->href_uri != NULL) {
+	if (emb->href_uri != NULL) {
 		uri = emb->href_uri;
 	} else {
 		uri = emb->current_uri;
@@ -1275,16 +1284,6 @@ on_popup_button_button_pressed (GtkToggleButton *button,
 	return FALSE;
 }
 
-static void
-on_popup_menu_unmap (GtkWidget *menu,
-		     TotemEmbedded *emb)
-{
-	GtkWidget *button;
-
-	button = GTK_WIDGET (gtk_builder_get_object (emb->xml, "popup_button"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
-}
-
 static char *
 resolve_redirect (const char *old_mrl, const char *mrl)
 {
@@ -1309,7 +1308,7 @@ resolve_redirect (const char *old_mrl, const char *mrl)
 static void
 on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 {
-	char *new_uri;
+	char *new_uri = NULL;
 
 	g_message ("stream uri: %s", emb->stream_uri ? emb->stream_uri : "(null)");
 	g_message ("current uri: %s", emb->current_uri ? emb->current_uri : "(null)");
@@ -1317,6 +1316,7 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 	g_message ("redirect: %s", mrl);
 
 	bacon_video_widget_close (emb->bvw);
+	update_fill (emb, -1.0);
 
 	/* If we don't have a relative URI */
 	if (strstr (mrl, "://") != NULL)
@@ -1336,7 +1336,7 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 	/* FIXME: clear playlist? or replace current entry? or add a new entry? */
 	/* FIXME: use totem_embedded_open_uri? */
 
-	totem_embedded_set_uri (emb, new_uri ? new_uri : mrl , emb->base_uri /* FIXME? */, FALSE);
+	totem_embedded_set_uri (emb, new_uri ? new_uri : mrl , emb->base_uri /* FIXME? */, emb->current_subtitle_uri, FALSE);
 
 	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 
@@ -1471,6 +1471,7 @@ on_eos_event (BaconVideoWidget *bvw, TotemEmbedded *emb)
 			emb->num_items = 1;
 			emb->is_browser_stream = FALSE;
 			bacon_video_widget_close (emb->bvw);
+			update_fill (emb, -1.0);
 			totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 		} else {
 			/* FIXME: should find a way to enable playback of the stream again without re-requesting it */
@@ -1486,10 +1487,12 @@ on_eos_event (BaconVideoWidget *bvw, TotemEmbedded *emb)
 					totem_embedded_play (emb, NULL);
 			} else {
 				bacon_video_widget_close (emb->bvw);
+				update_fill (emb, -1.0);
 				totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 			}
 		} else {
 			bacon_video_widget_close (emb->bvw);
+			update_fill (emb, -1.0);
 			totem_embedded_open_internal (emb, start_play, NULL /* FIXME? */);
 		}
 	} else if (emb->current) {
@@ -1550,7 +1553,7 @@ static void
 cb_vol (GtkWidget *val, gdouble value, TotemEmbedded *emb)
 {
 	bacon_video_widget_set_volume (emb->bvw, value);
-	totem_embedded_save_volume (emb, value);
+	totem_embedded_volume_changed (emb, value);
 }
 
 static void
@@ -1593,6 +1596,27 @@ on_buffering (BaconVideoWidget *bvw, guint percentage, TotemEmbedded *emb)
 //	g_message ("Buffering: %d %%", percentage);
 }
 
+static void
+update_fill (TotemEmbedded *emb, gdouble level)
+{
+	if (level < 0.0) {
+		gtk_range_set_show_fill_level (GTK_RANGE (emb->seek), FALSE);
+		gtk_range_set_show_fill_level (GTK_RANGE (emb->fs->seek), FALSE);
+	} else {
+		gtk_range_set_fill_level (GTK_RANGE (emb->seek), level * 65535.0f);
+		gtk_range_set_show_fill_level (GTK_RANGE (emb->seek), TRUE);
+
+		gtk_range_set_fill_level (GTK_RANGE (emb->fs->seek), level * 65535.0f);
+		gtk_range_set_show_fill_level (GTK_RANGE (emb->fs->seek), TRUE);
+	}
+}
+
+static void
+on_download_buffering (BaconVideoWidget *bvw, gdouble level, TotemEmbedded *emb)
+{
+	update_fill (emb, level);
+}
+
 static void
 property_notify_cb_volume (BaconVideoWidget *bvw,
 			   GParamSpec *spec,
@@ -1604,20 +1628,13 @@ property_notify_cb_volume (BaconVideoWidget *bvw,
 	
 	g_signal_handlers_block_by_func (emb->volume, cb_vol, emb);
 	gtk_scale_button_set_value (GTK_SCALE_BUTTON (emb->volume), volume);
-	totem_embedded_save_volume (emb, volume);
+	totem_embedded_volume_changed (emb, volume);
 	g_signal_handlers_unblock_by_func (emb->volume, cb_vol, emb);
 }
 
 static gboolean
 on_seek_start (GtkWidget *widget, GdkEventButton *event, TotemEmbedded *emb)
 {
-	/* HACK: we want the behaviour you get with the middle button, so we
-	 * mangle the event.  clicking with other buttons moves the slider in
-	 * step increments, clicking with the middle button moves the slider to
-	 * the location of the click.
-	 */
-	event->button = 2;
-
 	emb->seeking = TRUE;
 	totem_time_label_set_seeking (TOTEM_TIME_LABEL (emb->fs->time_label),
 				      TRUE);
@@ -1628,9 +1645,6 @@ on_seek_start (GtkWidget *widget, GdkEventButton *event, TotemEmbedded *emb)
 static gboolean
 cb_on_seek (GtkWidget *widget, GdkEventButton *event, TotemEmbedded *emb)
 {
-	/* HACK: see on_seek_start */
-	event->button = 2;
-
 	bacon_video_widget_seek (emb->bvw,
 		gtk_range_get_value (GTK_RANGE (widget)) / 65535, NULL);
 	totem_time_label_set_seeking (TOTEM_TIME_LABEL (emb->fs->time_label),
@@ -1670,7 +1684,7 @@ totem_embedded_action_volume_relative (TotemEmbedded *emb, double off_pct)
 	
 	vol = bacon_video_widget_get_volume (emb->bvw);
 	bacon_video_widget_set_volume (emb->bvw, vol + off_pct);
-	totem_embedded_save_volume (emb, vol + off_pct);
+	totem_embedded_volume_changed (emb, vol + off_pct);
 }
 
 static gboolean
@@ -1738,7 +1752,6 @@ totem_embedded_construct (TotemEmbedded *emb,
 {
 	GtkWidget *child, *container, *image;
 	GtkWidget *popup_button;
-	GtkWidget *menu;
 	BvwUseType type;
 	GError *err = NULL;
 	GConfClient *gc;
@@ -1786,6 +1799,16 @@ totem_embedded_construct (TotemEmbedded *emb,
 		emb->user_agent = NULL;
 	}
 
+        /* FIXMEchpe: this is just the initial value. I think in case e.g. we're doing
+         * a playlist, the playlist's URI should become the referrer?
+         */
+        g_print ("Referrer URI: %s\n", emb->referrer_uri);
+        if (emb->referrer_uri != NULL) {
+                bacon_video_widget_set_referrer (emb->bvw, emb->referrer_uri);
+                g_free (emb->referrer_uri);
+                emb->referrer_uri = NULL;
+        }
+
 	/* Fullscreen setup */
 	emb->fs_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_widget_realize (emb->fs_window);
@@ -1831,6 +1854,8 @@ totem_embedded_construct (TotemEmbedded *emb,
 			G_CALLBACK (on_tick), emb);
 	g_signal_connect (G_OBJECT (emb->bvw), "buffering",
 			  G_CALLBACK (on_buffering), emb);
+	g_signal_connect (G_OBJECT (emb->bvw), "download-buffering",
+			  G_CALLBACK (on_download_buffering), emb);
 
 	g_signal_connect (G_OBJECT (emb->bvw), "notify::volume",
 			  G_CALLBACK (property_notify_cb_volume), emb);
@@ -1886,7 +1911,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 	g_signal_connect (G_OBJECT (emb->volume), "value-changed",
 			  G_CALLBACK (cb_vol), emb);
 	bacon_video_widget_set_volume (emb->bvw, volume);
-	totem_embedded_save_volume (emb, volume);
+	totem_embedded_volume_changed (emb, volume);
 
 	emb->statusbar = TOTEM_STATUSBAR (gtk_builder_get_object (emb->xml, "statusbar"));
 	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (emb->statusbar), FALSE);
@@ -1944,10 +1969,6 @@ totem_embedded_construct (TotemEmbedded *emb,
 					     GTK_WINDOW (emb->window), emb);
 	g_assert (emb->menuxml);
 
-	menu = GTK_WIDGET (gtk_builder_get_object (emb->menuxml, "menu"));
-	g_signal_connect (G_OBJECT (menu), "unmap",
-			  G_CALLBACK (on_popup_menu_unmap), emb);
-
 	/* Set the logo and the button glow */
 	if (!emb->hidden && emb->autostart == FALSE) {
 		totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (emb->pp_button), TRUE);
@@ -1962,29 +1983,6 @@ totem_embedded_construct (TotemEmbedded *emb,
 	return TRUE;
 }
 
-static gboolean
-totem_embedded_construct_placeholder (TotemEmbedded *emb,
-			  GdkNativeWindow xid,
-			  int width,
-			  int height)
-{
-	GtkWidget *window, *label;
-
-	if (xid == 0)
-		return TRUE;
-
-	window = gtk_plug_new (xid);
-	/* FIXME why? */
-	gtk_widget_realize (window);
-
-	label = gtk_label_new ("Not yet supported");
-	gtk_container_add (GTK_CONTAINER (window), label);
-
-	gtk_widget_show_all (window);
-
-	return TRUE;
-}
-
 static gboolean
 totem_embedded_set_window (TotemEmbedded *embedded,
 			   const char *controls,
@@ -1995,22 +1993,13 @@ totem_embedded_set_window (TotemEmbedded *embedded,
 {
 	g_print ("Viewer: SetWindow XID %u size %d:%d\n", window, width, height);
 
-	if (embedded->type == TOTEM_PLUGIN_TYPE_COMPLEX) {
-		/* FIXME!!! */
-		if (strcmp (controls, "All") != 0 &&
-		    strcmp (controls, "ImageWindow") != 0) {
-			totem_embedded_construct_placeholder (embedded, (GdkNativeWindow) window, width, height);
-			return TRUE;
-		}
-	} else {
-		if (strcmp (controls, "All") != 0 &&
-		    strcmp (controls, "ImageWindow") != 0) {
-			g_set_error (error,
-				TOTEM_EMBEDDED_ERROR_QUARK,
-				TOTEM_EMBEDDED_SETWINDOW_UNSUPPORTED_CONTROLS,
-				"Unsupported controls '%s'", controls);
-			return FALSE;
-		}
+	if (strcmp (controls, "All") != 0 &&
+	    strcmp (controls, "ImageWindow") != 0) {
+		g_set_error (error,
+			     TOTEM_EMBEDDED_ERROR_QUARK,
+			     TOTEM_EMBEDDED_SETWINDOW_UNSUPPORTED_CONTROLS,
+			     "Unsupported controls '%s'", controls);
+		return FALSE;
 	}
 
 	if (embedded->window != NULL) {
@@ -2083,7 +2072,7 @@ entry_parsed (TotemPlParser *parser,
 	if (duration == 0)
 		return;
 
-	item = g_new0 (TotemPlItem, 1);
+	item = g_slice_new0 (TotemPlItem);
 	item->uri = g_strdup (uri);
 	item->title = g_strdup (g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE));
 	item->duration = duration;
@@ -2161,6 +2150,7 @@ totem_embedded_push_parser (gpointer data)
 static char *arg_user_agent = NULL;
 static char *arg_mime_type = NULL;
 static char **arg_remaining = NULL;
+static char *arg_referrer = NULL;
 static gboolean arg_no_controls = FALSE;
 static gboolean arg_statusbar = FALSE;
 static gboolean arg_hidden = FALSE;
@@ -2178,7 +2168,6 @@ parse_plugin_type (const gchar *option_name,
 {
 	const char types[TOTEM_PLUGIN_TYPE_LAST][12] = {
 		"gmp",
-		"complex",
 		"narrowspace",
 		"mully",
 		"cone"
@@ -2208,6 +2197,7 @@ static GOptionEntry option_entries [] =
 	{ TOTEM_OPTION_REPEAT, 0, 0, G_OPTION_ARG_NONE, &arg_repeat, NULL, NULL },
 	{ TOTEM_OPTION_NOAUTOSTART, 0, 0, G_OPTION_ARG_NONE, &arg_no_autostart, NULL, NULL },
 	{ TOTEM_OPTION_AUDIOONLY, 0, 0, G_OPTION_ARG_NONE, &arg_audioonly, NULL, NULL },
+        { TOTEM_OPTION_REFERRER, 0, 0, G_OPTION_ARG_STRING, &arg_referrer, NULL, NULL },
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY /* STRING? */, &arg_remaining, NULL },
 	{ NULL }
 };
@@ -2353,6 +2343,7 @@ int main (int argc, char **argv)
 	emb->audioonly = arg_audioonly;
 	emb->type = arg_plugin_type;
         emb->user_agent = arg_user_agent;
+        emb->referrer_uri = arg_referrer;
 
 	/* FIXME: register this BEFORE requesting the service name? */
 	dbus_g_connection_register_g_object
