@@ -2,7 +2,7 @@
  * 
  * Copyright © 2004-2006 Bastien Nocera <hadess@hadess.net>
  * Copyright © 2002 David A. Schleef <ds@schleef.org>
- * Copyright © 2006, 2007, 2008 Christian Persch
+ * Copyright © 2006, 2007, 2008, 2009 Christian Persch
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -64,8 +64,6 @@
 #include "totemGMPNetwork.h"
 #include "totemGMPPlayer.h"
 #include "totemGMPSettings.h"
-#elif defined(TOTEM_COMPLEX_PLUGIN)
-#include "totemComplexPlugin.h"
 #elif defined(TOTEM_NARROWSPACE_PLUGIN)
 #include "totemNarrowSpacePlugin.h"
 #elif defined(TOTEM_MULLY_PLUGIN)
@@ -101,8 +99,6 @@ static const totemPluginMimeEntry kMimeTypes[] = {
   { "application/x-ms-wmp", "wmp", "video/x-ms-wmv" },
   { "application/asx", "asx", "audio/x-ms-asx" },
   { "audio/x-ms-wma", "wma", NULL }
-#elif defined(TOTEM_COMPLEX_PLUGIN)
-  { "audio/x-pn-realaudio-plugin", "rpm", "audio/vnd.rn-realaudio" },
 #elif defined(TOTEM_NARROWSPACE_PLUGIN)
   { "video/quicktime", "mov", NULL },
   { "video/mp4", "mp4", NULL },
@@ -131,6 +127,7 @@ static const totemPluginMimeEntry kMimeTypes[] = {
   { "application/x-nsv-vp3-mp3", "nsv", "video/x-nsv" },
   { "video/flv", "flv", "application/x-flash-video" },
   { "application/x-totem-plugin", "", "Totem Multimedia plugin" },
+  { "audio/midi", "mid, midi", NULL },
 #else
 #error Unknown plugin type
 #endif
@@ -139,8 +136,6 @@ static const totemPluginMimeEntry kMimeTypes[] = {
 static const char kPluginDescription[] =
 #if defined(TOTEM_GMP_PLUGIN)
   "Windows Media Player Plug-in 10 (compatible; Totem)";
-#elif defined(TOTEM_COMPLEX_PLUGIN)
-  "Helix DNA Plugin: RealPlayer G2 Plug-In Compatible (compatible; Totem)";
 #elif defined(TOTEM_NARROWSPACE_PLUGIN)
   "QuickTime Plug-in " TOTEM_NARROWSPACE_VERSION;
 #elif defined(TOTEM_MULLY_PLUGIN)
@@ -167,34 +162,6 @@ static const char kPluginUserAgent[] =
   "";
 #endif
 
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-nsTArray<totemPlugin*> *totemPlugin::sPlugins;
-
-/* Keep the same order as totemPlugin::Control enum! */
-static const char *kControl[] = {
-	"All",
-	"ControlPanel",
-	"FFCtrl",
-	"HomeCtrl",
-	"ImageWindow",
-	"InfoPanel",
-	"InfoVolumePanel",
-	"MuteCtrl",
-	"MuteVolume",
-	"PauseButton",
-	"PlayButton",
-	"PlayOnlyButton",
-	"PositionField",
-	"PositionSlider",
-	"RWCtrl",
-	"StatusBar",
-	"StatusField",
-	"StopButton",
-	"TACCtrl",
-	"VolumeSlider",
-};
-#endif /* TOTEM_COMPLEX_PLUGIN */
-
 void*
 totemPlugin::operator new (size_t aSize) throw ()
 {
@@ -209,6 +176,7 @@ totemPlugin::operator new (size_t aSize) throw ()
 totemPlugin::totemPlugin (NPP aNPP)
 :	mNPP (aNPP),
         mMimeType (NULL),
+        mDocumentURI (NULL),
         mBaseURI (NULL),
         mSrcURI (NULL),
         mRequestBaseURI (NULL),
@@ -218,35 +186,15 @@ totemPlugin::totemPlugin (NPP aNPP)
 	mViewerFD (-1),
 	mWidth (-1),
 	mHeight (-1),
-#ifdef TOTEM_COMPLEX_PLUGIN
-	mAutoPlay (false),
-#else
 	mAutoPlay (true),
-#endif /* TOTEM_NARROWSPACE_PLUGIN */
 	mNeedViewer (true),
 	mState (TOTEM_STATE_STOPPED)
 {
         TOTEM_LOG_CTOR ();
-
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-	/* Add |this| to the global plugins list */
-	assert (sPlugins->IndexOf (this) == NoIndex); //, "WTF?");
-	if (!sPlugins->AppendElement (this)) {
-		D ("Couldn't maintain plugin list!");
-	}
-#endif /* TOTEM_COMPLEX_PLUGIN */
 }
 
 totemPlugin::~totemPlugin ()
 {
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-	/* Remove us from the plugins list */
-	NS_ASSERTION (sPlugins->IndexOf (this) != NoIndex, "WTF?");
-	sPlugins->RemoveElement (this);
-
-	TransferConsole ();
-#endif /* TOTEM_COMPLEX_PLUGIN */
-
         /* FIXMEchpe invalidate the scriptable object, or is that done automatically? */
 
 	if (mBusProxy) {
@@ -279,6 +227,7 @@ totemPlugin::~totemPlugin ()
         g_free (mMimeType);
 
         g_free (mSrcURI);
+        g_free (mDocumentURI);
         g_free (mBaseURI);
         g_free (mRequestURI);
         g_free (mRequestBaseURI);
@@ -294,16 +243,30 @@ totemPlugin::~totemPlugin ()
         TOTEM_LOG_DTOR ();
 }
 
+/* static */ void
+totemPlugin::QueueCommand (TotemQueueCommand *cmd)
+{
+	assert (mQueue);
+
+	g_queue_push_tail (mQueue, cmd);
+}
+
 /* public functions */
 
 void
 totemPlugin::Command (const char *aCommand)
 {
-	D ("Command '%s'", aCommand);
-
-	/* FIXME: queue the action instead */
-	if (!mViewerReady)
+	if (!mViewerReady) {
+		D("Queuing command '%s'", aCommand);
+		TotemQueueCommand *cmd;
+		cmd = g_new0 (TotemQueueCommand, 1);
+		cmd->type = TOTEM_QUEUE_TYPE_SET_STRING;
+		cmd->string = g_strdup (aCommand);
+		QueueCommand (cmd);
 		return;
+	}
+
+	D ("Command '%s'", aCommand);
 
 	assert (mViewerProxy);
 	dbus_g_proxy_call_no_reply (mViewerProxy,
@@ -354,11 +317,16 @@ totemPlugin::SetFullscreen (bool enabled)
 void
 totemPlugin::ClearPlaylist ()
 {
-	Dm ("ClearPlaylist");
-
-	/* FIXME: queue the action instead */
-	if (!mViewerReady)
+	if (!mViewerReady) {
+		Dm ("Queuing ClearPlaylist");
+		TotemQueueCommand *cmd;
+		cmd = g_new0 (TotemQueueCommand, 1);
+		cmd->type = TOTEM_QUEUE_TYPE_CLEAR_PLAYLIST;
+		QueueCommand (cmd);
 		return;
+	}
+
+	Dm ("ClearPlaylist");
 
 	assert (mViewerProxy);
 	dbus_g_proxy_call_no_reply (mViewerProxy,
@@ -368,28 +336,51 @@ totemPlugin::ClearPlaylist ()
 }
 
 int32_t
-totemPlugin::AddItem (const NPString& aURI)
+totemPlugin::AddItem (const NPString& aURI,
+		      const NPString& aTitle,
+		      const char *aSubtitle)
 {
+	Dm ("AddItem");
+
         if (!aURI.UTF8Characters || !aURI.UTF8Length)
                 return -1;
 
-        /* FIXMEchpe: resolve against mBaseURI or mSrcURI ?? */
+        char *uri = g_strndup (aURI.UTF8Characters, aURI.UTF8Length);
 
-	/* FIXME: queue the action instead */
-	if (!mViewerReady)
-		return false;
+	char *title;
+	if (aTitle.UTF8Characters && aURI.UTF8Length)
+		title = g_strndup (aTitle.UTF8Characters, aTitle.UTF8Length);
+	else
+		title = NULL;
+
+	if (!mViewerReady) {
+		D ("Queuing AddItem '%s' (title: '%s' sub: '%s')",
+		   uri, title ? title : "", aSubtitle ? aSubtitle : "");
+		TotemQueueCommand *cmd;
+		cmd = g_new0 (TotemQueueCommand, 1);
+		cmd->type = TOTEM_QUEUE_TYPE_ADD_ITEM;
+		cmd->add_item.uri = uri;
+		cmd->add_item.title = title;
+		cmd->add_item.subtitle = g_strdup (aSubtitle);
+		QueueCommand (cmd);
+		return 0;
+	}
+
+	D ("AddItem '%s' (title: '%s' sub: '%s')",
+	   uri, title ? title : "", aSubtitle ? aSubtitle : "");
 
 	assert (mViewerProxy);
 
-        char *uri = g_strndup (aURI.UTF8Characters, aURI.UTF8Length);
-	D ("AddItem '%s'", uri);
-
         dbus_g_proxy_call_no_reply (mViewerProxy,
 				    "AddItem",
+				    G_TYPE_STRING, mBaseURI,
 				    G_TYPE_STRING, uri,
+				    G_TYPE_STRING, title,
+				    G_TYPE_STRING, aSubtitle,
 				    G_TYPE_INVALID,
 				    G_TYPE_INVALID);
         g_free (uri);
+        g_free (title);
 
 	return 0;
 }
@@ -471,12 +462,6 @@ totemPlugin::Rate () const
 NPError
 totemPlugin::ViewerFork ()
 {
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-	/* Don't fork a viewer if we're going to use another one */
-	if (!mNeedViewer)
-		return NPERR_NO_ERROR;
-#endif /* TOTEM_COMPLEX_PLUGIN */
-
 	const char *userAgent = kPluginUserAgent;
 
 	if (*kPluginUserAgent == '\0') {
@@ -520,8 +505,6 @@ totemPlugin::ViewerFork ()
 	g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_PLUGIN_TYPE));
 #if defined(TOTEM_GMP_PLUGIN)
 	g_ptr_array_add (arr, g_strdup ("gmp"));
-#elif defined(TOTEM_COMPLEX_PLUGIN)
-	g_ptr_array_add (arr, g_strdup ("complex"));
 #elif defined(TOTEM_NARROWSPACE_PLUGIN)
 	g_ptr_array_add (arr, g_strdup ("narrowspace"));
 #elif defined(TOTEM_MULLY_PLUGIN)
@@ -537,6 +520,15 @@ totemPlugin::ViewerFork ()
 		g_ptr_array_add (arr, g_strdup (userAgent));
 	}
 
+        /* FIXMEchpe: This passes the document URI of the document the plugin is in
+         * as the HTTP referrer. I'm not at all sure this is the right URI! Need
+         * to check what exactly the various legacy plugins pass here.
+         */
+        if (mDocumentURI) {
+		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_REFERRER));
+		g_ptr_array_add (arr, g_strdup (mDocumentURI));
+        }
+
 	/* FIXME: remove this */
 	if (mMimeType) {
 		g_ptr_array_add (arr, g_strdup (DASHES TOTEM_OPTION_MIMETYPE));
@@ -622,6 +614,8 @@ totemPlugin::ViewerFork ()
 		return NPERR_GENERIC_ERROR;
 	}
 
+	mQueue = g_queue_new ();
+
 	/* Set mViewerFD nonblocking */
 	fcntl (mViewerFD, F_SETFL, O_NONBLOCK);
 
@@ -706,20 +700,8 @@ totemPlugin::ViewerSetup ()
 	} else {
 		ViewerSetWindow ();
 	}
-
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-	/* Notify all consoles */
-	uint32_t count = sPlugins->Length ();
-	for (uint32_t i = 0; i < count; ++i) {
-		totemPlugin *plugin = sPlugins->ElementAt (i);
-
-		if (plugin->mConsoleClassRepresentant == this)
-			plugin->UnownedViewerSetup ();
-	}
-#endif /* TOTEM_COMPLEX_PLUGIN */
 }
 
-
 void
 totemPlugin::ViewerCleanup ()
 {
@@ -796,11 +778,7 @@ totemPlugin::ViewerSetWindow ()
 					 ViewerSetWindowCallback,
 					 reinterpret_cast<void*>(this),
 					 NULL,
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-					 G_TYPE_STRING, mControls.get (),
-#else
 					 G_TYPE_STRING, "All",
-#endif
 					 G_TYPE_UINT, (guint) mWindow,
 					 G_TYPE_INT, mWidth,
 					 G_TYPE_INT, mHeight,
@@ -818,6 +796,56 @@ totemPlugin::ViewerReady ()
 
 	mViewerReady = true;
 
+	/* Unqueue any queued commands, before any
+	 * new ones come in */
+	TotemQueueCommand *cmd;
+
+	while ((cmd = (TotemQueueCommand *) g_queue_pop_head (mQueue)) != NULL) {
+		D("Popping command %d", cmd->type);
+		switch (cmd->type) {
+		case TOTEM_QUEUE_TYPE_CLEAR_PLAYLIST:
+			ClearPlaylist();
+			break;
+		case TOTEM_QUEUE_TYPE_ADD_ITEM:
+			assert (mViewerProxy);
+
+			D ("AddItem '%s' (base: '%s' title: '%s' sub: '%s')",
+			   cmd->add_item.uri, mBaseURI,
+			   cmd->add_item.title ? cmd->add_item.title : "",
+			   cmd->add_item.subtitle ? cmd->add_item.subtitle : "");
+			dbus_g_proxy_call_no_reply (mViewerProxy,
+						    "AddItem",
+						    G_TYPE_STRING, mBaseURI,
+						    G_TYPE_STRING, cmd->add_item.uri,
+						    G_TYPE_STRING, cmd->add_item.title,
+						    G_TYPE_STRING, cmd->add_item.subtitle,
+						    G_TYPE_INVALID,
+						    G_TYPE_INVALID);
+			g_free (cmd->add_item.uri);
+			g_free (cmd->add_item.title);
+			g_free (cmd->add_item.subtitle);
+			break;
+		case TOTEM_QUEUE_TYPE_SET_STRING:
+			assert (cmd->string);
+
+			if (g_str_equal (cmd->string, TOTEM_COMMAND_PLAY) ||
+			    g_str_equal (cmd->string, TOTEM_COMMAND_PAUSE) ||
+			    g_str_equal (cmd->string, TOTEM_COMMAND_STOP)) {
+				Command(cmd->string);
+			} else {
+				D("Unhandled queued string '%s'", cmd->string);
+			}
+			g_free (cmd->string);
+			break;
+		default:
+			D("Unhandled queued command type %d", cmd->type);
+		}
+
+		g_free (cmd);
+	}
+	g_queue_free (mQueue);
+	mQueue = NULL;
+
 	if (mAutoPlay) {
 		RequestStream (false);
 	} else {
@@ -1399,7 +1427,8 @@ totemPlugin::SetRealMimeType (const char *mimetype)
 {
 	for (uint32_t i = 0; i < G_N_ELEMENTS (kMimeTypes); ++i) {
 		if (strcmp (kMimeTypes[i].mimetype, mimetype) == 0) {
-			if (kMimeTypes[i].mime_alias != NULL) {
+			if (kMimeTypes[i].mime_alias != NULL &&
+			    strchr (kMimeTypes[i].mime_alias, '/') != NULL) {
 				mMimeType = g_strdup (kMimeTypes[i].mime_alias);
 			} else {
 				mMimeType = g_strdup (mimetype);
@@ -1689,201 +1718,6 @@ totemPlugin::ParseURLExtensions (const char* str,
 
 #endif /* TOTEM_NARROWSPACE_PLUGIN */
 
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-
-totemPlugin *
-totemPlugin::FindConsoleClassRepresentant ()
-{
-	/* FIXME: this treats "master" incorrectly */
-	if (!mSrcURI ||
-	    mConsole.IsEmpty () ||
-	    mConsole.Equals (NS_LITERAL_CSTRING ("_unique")) ||
-	    mConsole.Equals (NS_LITERAL_CSTRING ("_master"))) {
-		D ("We're the representant for the console class");
-		return NULL;
-	}
-
-	totemPlugin *representant = NULL;
-
-	/* Try to find a the representant of the console class */
-	uint32_t count = sPlugins->Length ();
-	for (uint32_t i = 0; i < count; ++i) {
-		totemPlugin *plugin = sPlugins->ElementAt (i);
-
-		bool equal = false;
-		/* FIXME: is this correct? Maybe we should use the toplevel document
-		 * to allow frames (and check the frames for same-origin, obviously) ?
-		 */
-		if (plugin != this &&
-		    plugin->mPluginOwnerDocument == mPluginOwnerDocument &&
-		    mConsole.Equals (plugin->mConsole) &&
-		    plugin->mSrcURI &&
-		    NS_SUCCEEDED (plugin->mSrcURI->Equals (mSrcURI, &equal)) &&
-		    equal) {
-			if (plugin->mConsoleClassRepresentant) {
-				representant = plugin->mConsoleClassRepresentant;
-			} else {
-				representant = plugin;
-			}
-			break;
-		}
-	}
-
-	D ("Representant for the console class is %p", (void*) representant);
-
-	return representant;
-}
-
-bool
-totemPlugin::SetConsole (const char * aConsole)
-{
-	/* Can't change console group */
-	if (mConsole) {
-// 		return NS_ERROR_ALREADY_INITIALIZED;
-          // FIXMEchpe set exception
-          return false;
-        }
-
-	/* FIXME: we might allow this, and kill the viewer instead.
-	 * Or maybe not spawn the viewer if we don't have a console yet?
-	 */
-	if (mViewerPID)
-		return false; // FIXMEchpe throw exception
-
-        mConsole = g_strdup (aConsole);
-
-// 	assert (mConsoleClassRepresentant == NULL, "Already have a representant");
-
-	mConsoleClassRepresentant = FindConsoleClassRepresentant ();
-	mNeedViewer = (NULL == mConsoleClassRepresentant);
-
-	return true;
-}
-
-void
-totemPlugin::TransferConsole ()
-{
-	/* Find replacement representant */
-	totemPlugin *representant = NULL;
-
-	uint32_t i, count = sPlugins->Length ();
-	for (i = 0; i < count; ++i) {
-		totemPlugin *plugin = sPlugins->ElementAt (i);
-
-		if (plugin->mConsoleClassRepresentant == this) {
-			representant = plugin;
-			break;
-		}
-	}
-
-	/* If there are no other elements in this console class, there's nothing to do */
-	if (!representant)
-		return;
-
-	D ("Transferring console from %p to %p", (void*) this, (void*) representant);
-
-	/* Store new representant in the plugins */
-	representant->mConsoleClassRepresentant = NULL;
-	/* We can start at i since we got out when we found the first one in the loop above */
-	for ( ; i < count; ++i) {
-		totemPlugin *plugin = sPlugins->ElementAt (i);
-
-		if (plugin->mConsoleClassRepresentant == this)
-			plugin->mConsoleClassRepresentant = representant;
-	}
-
-	/* Now transfer viewer ownership */
-#if 0
-	if (mScriptable) {
-		assert (!representant->mScriptable, "WTF");
-		representant->mScriptable = mScriptable;
-		mScriptable->SetPlugin (representant);
-		mScriptable = NULL;
-	}
-#endif
-
-	representant->mNeedViewer = true;
-
-	representant->mViewerPID = mViewerPID;
-	mViewerPID = 0;
-
-	representant->mViewerFD = mViewerFD;
-	mViewerFD = -1;
-
-	representant->mViewerBusAddress = g_strdup (mViewerBusAddress);
-	representant->mViewerServiceName = g_strdup (mViewerServiceName);
-
-	/* FIXME correct condition? */
-	if (mViewerSetUp)
-		representant->ViewerSetup ();
-}
-
-void
-totemPlugin::UnownedViewerSetup ()
-{
-	/* already set up */
-	if (mUnownedViewerSetUp)
-		return;
-
-	mUnownedViewerSetUp = true;
-
-	D ("UnownedViewerSetup");
-
-	assert (mConsoleClassRepresentant); /* We own the viewer!? */
-
-	UnownedViewerSetWindow ();
-}
-
-void
-totemPlugin::UnownedViewerSetWindow ()
-{
-	if (mWindowSet || mWindow == 0)
-		return;
-
-	if (!mUnownedViewerSetUp) {
-		D ("No unowned viewer yet, deferring SetWindow");
-		return;
-	}
-
-	assert (mConsoleClassRepresentant); /* We own the viewer! */
-
-	assert (mConsoleClassRepresentant->mViewerProxy);
-
-	/* FIXME: do we need a reply callback? */
-	dbus_g_proxy_call_no_reply (mConsoleClassRepresentant->mViewerProxy,
-				    "SetWindow",
-				    G_TYPE_STRING, mControls.get (),
-				    G_TYPE_UINT, (guint) mWindow,
-				    G_TYPE_INT, mWidth,
-				    G_TYPE_INT, mHeight,
-				    G_TYPE_INVALID);
-
-	mWindowSet = true;
-}
-
-void
-totemPlugin::UnownedViewerUnsetWindow ()
-{
-	if (!mWindowSet || mWindow == 0)
-		return;
-
-	if (!mUnownedViewerSetUp)
-		return;
-
-	assert (mConsoleClassRepresentant); /* We own the viewer! */
-
-	assert (mConsoleClassRepresentant->mViewerProxy);
-
-	dbus_g_proxy_call_no_reply (mConsoleClassRepresentant->mViewerProxy,
-				    "UnsetWindow",
-				    G_TYPE_UINT, (guint) mWindow,
-				    G_TYPE_INVALID);
-
-	mWindowSet = false;
-}
-
-#endif /* TOTEM_COMPLEX_PLUGIN */
-
 /* Plugin glue functions */
 
 NPError
@@ -1911,17 +1745,32 @@ totemPlugin::Init (NPMIMEType mimetype,
 		return NPERR_GENERIC_ERROR;
 	}
 
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-        /* FIXMEchpe untested; test this! */
+        /* FIXMEchpe: should use totemNPObjectWrapper + getter_Retains(),
+         * but that causes a crash somehow, deep inside libxul...
+         */
+        totemNPVariantWrapper ownerDocument;
         if (!NPN_GetProperty (mNPP,
                               mPluginElement,
                               NPN_GetStringIdentifier ("ownerDocument"),
-                              getter_Retains (mPluginOwnerDocument)) ||
-            mPluginOwnerDocument.IsNull ()) {
+                              getter_Copies (ownerDocument)) ||
+            !ownerDocument.IsObject ()) {
 		Dm ("Failed to get the plugin element's ownerDocument");
 		return NPERR_GENERIC_ERROR;
 	}
-#endif /* TOTEM_COMPLEX_PLUGIN */
+
+        totemNPVariantWrapper docURI;
+        if (!NPN_GetProperty (mNPP,
+                              ownerDocument.GetObject(),
+                              NPN_GetStringIdentifier ("documentURI"),
+                              getter_Copies (docURI)) ||
+            !docURI.IsString ()) {
+                Dm ("Failed to get the document URI");
+                return NPERR_GENERIC_ERROR;
+        }
+  
+        /* FIXMEchpe: need to resolve this against any base URIs ? */
+        mDocumentURI = g_strndup (docURI.GetString(), docURI.GetStringLen());
+        D ("Document URI is '%s'", mDocumentURI ? mDocumentURI : "");
 
 	/* We'd like to get the base URI of our DOM element as a nsIURI,
 	 * but there's no frozen method to do so (nsIContent/nsINode isn't available
@@ -2145,43 +1994,6 @@ totemPlugin::Init (NPMIMEType mimetype,
 		mControllerHidden = true;
 #endif
 
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-	value = (const char *) g_hash_table_lookup (args, "console");
-	if (value) {
-		rv = SetConsole (nsDependentCString (value));
-		if (NS_FAILED (rv))
-			return NPERR_GENERIC_ERROR;
-	}
-
-	const char *kControls[] = {
-		"All",
-		"ControlPanel",
-		"FFCtrl",
-		"HomeCtrl",
-		"ImageWindow",
-		"InfoPanel",
-		"InfoVolumePanel",
-		"MuteCtrl",
-		"MuteVolume",
-		"PauseButton",
-		"PlayButton",
-		"PlayOnlyButton",
-		"PositionField",
-		"PositionSlider",
-		"RWCtrl",
-		"StatusBar",
-		"StatusField",
-		"StopButton",
-		"TACCtrl",
-		"VolumeSlider",
-	};
-	uint32_t control = GetEnumIndex (args, "controls",
-				         kControls, G_N_ELEMENTS (kControls),
-					 0);
-	mControls = kControls[control];
-
-#endif /* TOTEM_COMPLEX_PLUGIN */
-
 #ifdef TOTEM_GMP_PLUGIN
 	/* uimode is either invisible, none, mini, or full
 	 * http://windowssdk.msdn.microsoft.com/en-us/library/aa392439(VS.80).aspx */
@@ -2248,10 +2060,6 @@ totemPlugin::Init (NPMIMEType mimetype,
 	D ("mHref: %s", mHref ? mHref : "");
 	D ("mTarget: %s", mTarget ? mTarget : "");
 #endif /* TOTEM_NARROWSPACE_PLUGIN */
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-	D ("mConsole: %s", mConsole.get ());
-	D ("mControls: %s", mControls.get ());
-#endif /* TOTEM_COMPLEX_PLUGIN */
 
 	g_hash_table_destroy (args);
 
@@ -2647,8 +2455,6 @@ totemPlugin::GetNPObject (ObjectEnum which)
     case eLastNPObject:
       g_assert_not_reached ();
   }
-#elif defined(TOTEM_COMPLEX_PLUGIN)
-  npclass = totemComplexPluginNPClass::Instance();
 #elif defined(TOTEM_NARROWSPACE_PLUGIN)
   npclass = totemNarrowSpacePlayerNPClass::Instance();
 #elif defined(TOTEM_MULLY_PLUGIN)
@@ -2710,36 +2516,17 @@ totemPlugin::GetScriptableNPObject (void *_retval)
 /* static */ NPError
 totemPlugin::Initialise ()
 {
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-	sPlugins = new nsTArray<totemPlugin*> (32);
-	if (!sPlugins)
-		return NPERR_OUT_OF_MEMORY_ERROR;
-#endif /* TOTEM_COMPLEX_PLUGIN */
-
 	return NPERR_NO_ERROR;
 }
 
 /* static */ NPError
 totemPlugin::Shutdown ()
 {
-#if defined(TOTEM_COMPLEX_PLUGIN) && defined(HAVE_NSTARRAY_H)
-	if (sPlugins) {
-		if (!sPlugins->IsEmpty ()) {
-			D ("WARNING: sPlugins not empty on shutdown, count: %d", sPlugins->Length ());
-		}
-
-		delete sPlugins;
-		sPlugins = NULL;
-	}
-#endif /* TOTEM_COMPLEX_PLUGIN */
-
 #if defined(TOTEM_GMP_PLUGIN)
         totemGMPPlayerNPClass::Shutdown ();
         totemGMPControlsNPClass::Shutdown ();
         totemGMPNetworkNPClass::Shutdown ();
         totemGMPSettingsNPClass::Shutdown ();
-#elif defined(TOTEM_COMPLEX_PLUGIN)
-	totemComplexPluginNPClass::Shutdown ();
 #elif defined(TOTEM_NARROWSPACE_PLUGIN)
         totemNarrowSpacePlayerNPClass::Shutdown ();
 #elif defined(TOTEM_MULLY_PLUGIN)
