@@ -48,7 +48,6 @@
 #include "totem-private.h"
 
 struct _TotemCellRendererVideoPrivate {
-	gboolean dispose_has_run;
 	gchar *title;
 	GdkPixbuf *thumbnail;
 	PangoAlignment alignment;
@@ -67,6 +66,7 @@ enum {
 static void totem_cell_renderer_video_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static void totem_cell_renderer_video_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void totem_cell_renderer_video_dispose (GObject *object);
+static void totem_cell_renderer_video_finalize (GObject *object);
 static void totem_cell_renderer_video_get_size (GtkCellRenderer *cell, GtkWidget *widget, GdkRectangle *cell_area, gint *x_offset, gint *y_offset, gint *width, gint *height);
 static void totem_cell_renderer_video_render (GtkCellRenderer *cell, GdkDrawable *window, GtkWidget *widget, GdkRectangle *background_area, GdkRectangle *cell_area, GdkRectangle *expose_area, GtkCellRendererState flags);
 
@@ -102,6 +102,7 @@ totem_cell_renderer_video_class_init (TotemCellRendererVideoClass *klass)
 	object_class->set_property = totem_cell_renderer_video_set_property;
 	object_class->get_property = totem_cell_renderer_video_get_property;
 	object_class->dispose = totem_cell_renderer_video_dispose;
+	object_class->finalize = totem_cell_renderer_video_finalize;
 	renderer_class->get_size = totem_cell_renderer_video_get_size;
 	renderer_class->render = totem_cell_renderer_video_render;
 
@@ -149,7 +150,6 @@ static void
 totem_cell_renderer_video_init (TotemCellRendererVideo *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, TOTEM_TYPE_CELL_RENDERER_VIDEO, TotemCellRendererVideoPrivate);
-	self->priv->dispose_has_run = FALSE;
 	self->priv->title = NULL;
 	self->priv->thumbnail = NULL;
 	self->priv->alignment = PANGO_ALIGN_CENTER;
@@ -164,19 +164,25 @@ totem_cell_renderer_video_dispose (GObject *object)
 {
 	TotemCellRendererVideo *self = TOTEM_CELL_RENDERER_VIDEO (object);
 
-	/* Make sure we only run once */
-	if (self->priv->dispose_has_run)
-		return;
-	self->priv->dispose_has_run = TRUE;
-
-	g_free (self->priv->title);
 	if (self->priv->thumbnail != NULL)
 		g_object_unref (self->priv->thumbnail);
+	self->priv->thumbnail = NULL;
 
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (totem_cell_renderer_video_parent_class)->dispose (object);
 }
 
+static void
+totem_cell_renderer_video_finalize (GObject *object)
+{
+	TotemCellRendererVideo *self = TOTEM_CELL_RENDERER_VIDEO (object);
+
+	g_free (self->priv->title);
+
+	/* Chain up to the parent class */
+	G_OBJECT_CLASS (totem_cell_renderer_video_parent_class)->finalize (object);
+}
+
 static void
 totem_cell_renderer_video_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
 {
@@ -187,7 +193,7 @@ totem_cell_renderer_video_set_property (GObject *object, guint property_id, cons
 		case PROP_THUMBNAIL:
 			if (priv->thumbnail != NULL)
 				g_object_unref (priv->thumbnail);
-			priv->thumbnail = (GdkPixbuf*) g_value_dup_object (value);
+			priv->thumbnail = GDK_PIXBUF (g_value_dup_object (value));
 			break;
 		case PROP_TITLE:
 			g_free (priv->title);
@@ -236,7 +242,7 @@ get_size (GtkCellRenderer *cell,
 	  GdkRectangle *title_area,
 	  GdkRectangle *thumbnail_area)
 {
-	TotemCellRendererVideoPrivate *priv = TOTEM_CELL_RENDERER_VIDEO_GET_PRIVATE (cell);
+	TotemCellRendererVideoPrivate *priv = TOTEM_CELL_RENDERER_VIDEO (cell)->priv;
 	guint pixbuf_width = 0;
 	guint pixbuf_height = 0;
 	guint title_width, title_height;
@@ -266,17 +272,16 @@ get_size (GtkCellRenderer *cell,
 	if (priv->thumbnail != NULL)
 		pango_font_description_set_weight (font_desc, PANGO_WEIGHT_BOLD);
 	context = gtk_widget_get_pango_context (widget);
-	metrics = pango_context_get_metrics (context,
-				font_desc,
-				pango_context_get_language (context));
+	metrics = pango_context_get_metrics (context, font_desc, pango_context_get_language (context));
 
 	if (cell_area)
 		title_width = cell_area->width;
-	else
+	else if (cell->width != -1)
 		title_width = cell->width;
+	else
+		title_width = pixbuf_width;
 
-	title_height = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics) +
-				pango_font_metrics_get_descent (metrics));
+	title_height = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics) + pango_font_metrics_get_descent (metrics));
 
 	pango_font_metrics_unref (metrics);
 	pango_font_description_free (font_desc);
@@ -288,8 +293,7 @@ get_size (GtkCellRenderer *cell,
 	if (draw_area) {
 		if (cell_area && calc_width > 0 && calc_height > 0) {
 			draw_area->x = (((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) ?
-				(1.0 - cell->xalign) : cell->xalign) * 
-				(cell_area->width - calc_width));
+				(1.0 - cell->xalign) : cell->xalign) * (cell_area->width - calc_width));
 			draw_area->x = MAX (draw_area->x, 0);
 			draw_area->y = (cell->yalign * (cell_area->height - calc_height));
 			draw_area->y = MAX (draw_area->y, 0);
@@ -298,15 +302,11 @@ get_size (GtkCellRenderer *cell,
 			draw_area->y = 0;
 		}
 
+		draw_area->x += cell->xpad;
+		draw_area->y += cell->ypad;
 		draw_area->width = calc_width;
 		draw_area->height = calc_height;
 
-		/*if (cell_area) {
-			g_message ("Cell area: X: %i, Y: %i, W: %i, H: %i", cell_area->x, cell_area->y, cell_area->width, cell_area->height);
-			g_message ("X-align: %f, Y-align: %f", cell->xalign, cell->yalign);
-		}
-		g_message ("Draw area: X: %i, Y: %i, W: %i, H: %i", draw_area->x, draw_area->y, draw_area->width, draw_area->height);*/
-
 		if (title_area) {
 			if (cell_area) {
 				title_area->width = cell_area->width;
@@ -321,8 +321,6 @@ get_size (GtkCellRenderer *cell,
 				title_area->y = draw_area->y + pixbuf_height + cell->ypad;
 			else
 				title_area->y = draw_area->y;
-
-			/*g_message ("Title area: X: %i, Y: %i, W: %i, H: %i", title_area->x, title_area->y, title_area->width, title_area->height);*/
 		}
 
 		if (pixbuf_height > 0 && thumbnail_area) {
@@ -333,11 +331,7 @@ get_size (GtkCellRenderer *cell,
 			thumbnail_area->y = draw_area->y;
 			thumbnail_area->width = cell->xpad * 2 + pixbuf_width;
 			thumbnail_area->height = pixbuf_height;
-
-			/*g_message ("Thumbnail area: X: %i, Y: %i, W: %i, H: %i", thumbnail_area->x, thumbnail_area->y, thumbnail_area->width, thumbnail_area->height);*/
 		}
-
-		/*g_message ("---");*/
 	}
 }
 
@@ -351,6 +345,7 @@ totem_cell_renderer_video_get_size (GtkCellRenderer *cell,
 				    gint *height)
 {
 	GdkRectangle draw_area;
+
 	get_size (cell, widget, cell_area, &draw_area, NULL, NULL);
 
 	if (x_offset)
@@ -372,7 +367,7 @@ totem_cell_renderer_video_render (GtkCellRenderer *cell,
 				  GdkRectangle *expose_area,
 				  GtkCellRendererState flags)
 {
-	TotemCellRendererVideoPrivate *priv = TOTEM_CELL_RENDERER_VIDEO_GET_PRIVATE (cell);
+	TotemCellRendererVideoPrivate *priv = TOTEM_CELL_RENDERER_VIDEO (cell)->priv;
 	GdkPixbuf *pixbuf;
 	GdkRectangle draw_rect;
 	GdkRectangle draw_area;
@@ -380,28 +375,24 @@ totem_cell_renderer_video_render (GtkCellRenderer *cell,
 	GdkRectangle thumbnail_area;
 	cairo_t *cr;
 	PangoLayout *layout;
-	PangoFontDescription *desc;
 	GtkStateType state;
 
 	get_size (cell, widget, cell_area, &draw_area, &title_area, &thumbnail_area);
 
-	draw_area.x += cell_area->x + cell->xpad;
-	draw_area.y += cell_area->y + cell->ypad;
+	draw_area.x += cell_area->x;
+	draw_area.y += cell_area->y;
 	draw_area.width -= cell->xpad * 2;
 	draw_area.height -= cell->ypad * 2;
 
 	if (!gdk_rectangle_intersect (cell_area, &draw_area, &draw_rect) ||
-				!gdk_rectangle_intersect (expose_area, &draw_rect, &draw_rect))
+	    !gdk_rectangle_intersect (expose_area, &draw_rect, &draw_rect))
 		return;
 
 	/* Sort out the thumbnail */
 	if (priv->thumbnail != NULL)
 		pixbuf = priv->thumbnail;
 	else if (priv->use_placeholder == TRUE)
-		pixbuf = gtk_widget_render_icon (widget,
-					GTK_STOCK_MISSING_IMAGE,
-					GTK_ICON_SIZE_DIALOG,
-					NULL);
+		pixbuf = gtk_widget_render_icon (widget, GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG, NULL);
 	else
 		pixbuf = NULL;
 
@@ -428,34 +419,29 @@ totem_cell_renderer_video_render (GtkCellRenderer *cell,
 
 	/* Draw the title */
 	layout = gtk_widget_create_pango_layout (widget, priv->title);
-	desc = pango_font_description_copy_static (widget->style->font_desc);
-	if (pixbuf != NULL)
+	if (pixbuf != NULL) {
+		PangoFontDescription *desc = pango_font_description_copy_static (widget->style->font_desc);
 		pango_font_description_set_weight (desc, PANGO_WEIGHT_BOLD);
+		pango_layout_set_font_description (layout, desc);
+		pango_font_description_free (desc);
+	}
 
-	pango_layout_set_font_description (layout, desc);
 	pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_END);
 	pango_layout_set_width (layout, title_area.width * PANGO_SCALE);
 	pango_layout_set_alignment (layout, priv->alignment);
 
-	gtk_paint_layout (widget->style,
-				window,
-				state,
-				TRUE,
-				expose_area,
-				widget,
-				"cellrenderervideotitle",
-				cell_area->x + title_area.x + cell->xpad,
-				cell_area->y + title_area.y + cell->ypad,
-				layout);
+	gtk_paint_layout (widget->style, window, state, TRUE, expose_area, widget, "cellrenderervideotitle",
+			  cell_area->x + title_area.x,
+			  cell_area->y + title_area.y,
+			  layout);
 
-	pango_font_description_free (desc);
 	g_object_unref (layout);
 
 	/* Draw the thumbnail */
 	if (pixbuf != NULL) {
 		cr = gdk_cairo_create (window);
 
-		gdk_cairo_set_source_pixbuf (cr, pixbuf, cell_area->x + thumbnail_area.x + cell->xpad, cell_area->y + thumbnail_area.y + cell->ypad);
+		gdk_cairo_set_source_pixbuf (cr, pixbuf, cell_area->x + thumbnail_area.x, cell_area->y + thumbnail_area.y);
 		gdk_cairo_rectangle (cr, &draw_rect);
 		cairo_fill (cr);
 
